/*
   This program demonstrates how to use the "shape library".
*/
import java.awt.Color;

public class ShapeModels
{
   public static void main(String[] args)
   {
      int k = 0; // number of triangles in the triangle fans at the top and bottom
      int n = 0; // number of triangle strips between the top and bottom

      // Check for two command line arguments.
      if (args.length >=2)
      {
         k = Integer.parseInt( args[0] );
         n = Integer.parseInt( args[1] );
      }
      else
      {
         System.err.printf("Usage: %s <k> <n>", ShapeModels.class.getName());
         System.exit(-1);;
      }

      // Create several Model objects.
      Model m1 = new TriangleModel(3, 3, 3, Color.red);
      m1.doBackFaceCulling = false; // try setting this true
      m1.modelTranslate(-3, -3, -3);

      Model m2 = new CubeModel(1, 1, 1);
      for (Triangle t : m2.triangleList)
      {
         t.v[0].setColorRandom();
         t.setColor( t.v[0].getColor() );
      }
      m2.modelTranslate(-4, 0, 0);

      Model m3 = new SphereModel(k, n, 1);
      for (Triangle t : m3.triangleList)
      {
         t.v[0].setColorRandom();
         t.setColor( t.v[0].getColor() );
      }
      m3.modelTranslate(-2, 3, 0);

      Model m4 = new ConeModel(4, 4, 1, 2);
      for (Triangle t : m4.triangleList)
      {
         t.v[0].setColorRandom();
         t.setColor( t.v[0].getColor() );
      }

      Model m5 = new CylinderModel(k, n, 1, 4);
      for (Triangle t : m5.triangleList)
      {
         t.v[0].setColorRandom();
         t.setColor( t.v[0].getColor() );
      }
      m5.modelTranslate(3, 0, 0);

      Model m6 = new PanelModel(-5, 5, -5, 5, Color.white, Color.black);
      m6.doLighting = false;
      m6.modelTranslate(0, -5, 0);

      // Create the Scene object that we shall render
      Scene scene = new Scene();

      // Add the models to the Scene.
      scene.modelList.add(m1);
      scene.modelList.add(m2);
      scene.modelList.add(m3);
      scene.modelList.add(m4);
      scene.modelList.add(m5);
      scene.modelList.add(m6);

      // Give the Scene a view matrix.
      scene.view2Identity();

      // Give the scene a projection matrix.
      scene.proj2Identity();

      // Set up the scene's view frustum.
      double fov    = 60.0;
      double aspect = 1.0;
      double near   = 1.0;
      double far    = 20.0;
      scene.projPerspective(fov, aspect, near, far);

      // Set up the scene's "camera".
      double eyex=0,    eyey=0,    eyez=10;
      double centerx=0, centery=0, centerz=0;
      double upx=0,     upy=1,     upz=0;
      scene.viewLookAt(eyex,    eyey,    eyez,
                       centerx, centery, centerz,
                       upx,     upy,     upz);

      //System.err.println( scene.toString(3) );

      // Create a framebuffer to render our scene into.
      int vp_width  = 512;
      int vp_height = 512;
      FrameBuffer fb = new FrameBuffer(vp_width, vp_height);
      // Give the framebuffer a nice background color.
      fb.clearFB(java.awt.Color.darkGray);
      // Render our scene into the frame buffer.
      SceneRender.renderScene(scene, fb);
      // Save the resulting image in a file.
      fb.dumpFB2File( String.format("ShapesColor_Frame%02d.ppm", 0) );

      // Spin the scene 360 degrees.
      for (int i = 1; i <= 36; i++)
      {
         // Rotate the scene by 10 degrees.
         scene.viewRotate(10, 0, 1, 0);
         // Render again.
         fb.clearFB(java.awt.Color.darkGray);
         SceneRender.renderScene(scene, fb);
         fb.dumpFB2File(String.format("ShapesColor_Frame%02d.ppm", i));
      }
   }
}